package com.tencent.cloud.rn.callkit;

import android.content.Context;
import androidx.annotation.NonNull;

import java.util.ArrayList;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.tencent.cloud.rn.callkit.utils.ParamsTransformer;
import com.tencent.qcloud.tuicore.TUILogin;
import com.tencent.qcloud.tuicore.interfaces.TUICallback;
import com.tencent.qcloud.tuikit.TUICommonDefine;
import com.tencent.qcloud.tuikit.tuicallengine.TUICallEngine;
import com.tencent.qcloud.tuikit.tuicallkit.TUICallKit;
import com.tencent.cloud.rn.callkit.listener.CallEventDispatcher;

@ReactModule(name = TUICallKitReactNativeSdkModule.NAME)
public class TUICallKitReactNativeSdkModule extends ReactContextBaseJavaModule {
  public static final String NAME = "CallUikit";

  private final Context mContext;
  private CallEventDispatcher mListener;
  private int mListenerCount = 0;

  @NonNull
  @Override
  public String getName() {
    return NAME;
  }

  public TUICallKitReactNativeSdkModule(ReactApplicationContext context) {
    super(context);
    mListener = new CallEventDispatcher(context);
    mContext = context.getApplicationContext();
  }

  @ReactMethod
  public void login(int sdkAppId, String userId, String userSig, Callback onSuccess, Callback onError) {
    TUILogin.login(mContext, sdkAppId, userId, userSig, new TUICallback() {
      @Override
      public void onSuccess() {
        onSuccess.invoke("Success.");
      }

      @Override
      public void onError(int errorCode, String errorMessage) {
        onError.invoke(errorCode, errorMessage);
      }
    });
  }

  @ReactMethod
  public void logout(Callback onSuccess, Callback onError) {
    TUILogin.logout(new TUICallback() {
      @Override
      public void onSuccess() {
        destroyInstance();
        onSuccess.invoke("Success.");
      }

      @Override
      public void onError(int errorCode, String errorMessage) {
        destroyInstance();
        onError.invoke(errorCode, errorMessage);
      }
    });
  }

  // Will be called when this module's first listener is add.
  @ReactMethod
  public void addListener(String eventName) {
    if (mListenerCount == 0) {
      TUICallEngine.createInstance(mContext).addObserver(mListener);
    }
    mListenerCount += 1;
  }

  // Will be called when this module's last listener is removed.
  @ReactMethod
  public void removeListeners(Integer count) {
    mListenerCount -= count;
    if (mListenerCount == 0) {
      TUICallEngine.createInstance(mContext).removeObserver(mListener);
    }
  }

  @ReactMethod
  public void setSelfInfo(String userName, String avatar, Callback onSuccess, Callback onError) {

    TUICallKit.createInstance(mContext).setSelfInfo(userName, avatar, new TUICommonDefine.Callback() {
      @Override
      public void onSuccess() {
        onSuccess.invoke("Success.");
      }

      @Override
      public void onError(int code, String message) {
        onError.invoke(code, message);
      }
    });
  }

  @ReactMethod
  public void call(String userId, Integer mediaType, ReadableMap callParamsMap, Callback onSuccess, Callback onError) {
    TUICallKit.createInstance(mContext).call(userId, ParamsTransformer.transMediaType(mediaType), ParamsTransformer.transCallParams(callParamsMap), new TUICommonDefine.Callback() {
      @Override
      public void onSuccess() {
        onSuccess.invoke("Success.");
      }

      @Override
      public void onError(int code, String message) {
        onError.invoke(code, message);
      }
    });
  }

  @ReactMethod
  public void groupCall(String groupId, ReadableArray userIdReadableArray, Integer mediaType, ReadableMap callParamsMap, Callback success, Callback error) {
    ArrayList<String> userIdList = new ArrayList<>();
    if (userIdReadableArray != null) {
      for (Object userId : userIdReadableArray.toArrayList()) {
        userIdList.add((String) userId);
      }
    }

    TUICallKit.createInstance(mContext).groupCall(groupId, userIdList, ParamsTransformer.transMediaType(mediaType), ParamsTransformer.transCallParams(callParamsMap), new TUICommonDefine.Callback() {
      @Override
      public void onSuccess() {
        success.invoke("Success.");
      }

      @Override
      public void onError(int code, String message) {
        error.invoke(code, message);
      }
    });
  }

  @ReactMethod
  public void joinInGroupCall(String strRoomId, String groupId, Integer mediaType) {
    TUICommonDefine.RoomId roomId = new TUICommonDefine.RoomId();
    roomId.strRoomId = strRoomId;
    TUICallKit.createInstance(mContext).joinInGroupCall(roomId, groupId, ParamsTransformer.transMediaType(mediaType));
  }

  @ReactMethod
  public void setCallingBell(String filePath) {
    TUICallKit.createInstance(mContext).setCallingBell(filePath);
  }

  @ReactMethod
  public void enableMuteMode(boolean enable) {
    TUICallKit.createInstance(mContext).enableMuteMode(enable);
  }

  @ReactMethod
  public void enableVirtualBackground(boolean enable) {
    TUICallKit.createInstance(mContext).enableVirtualBackground(enable);
  }

  @ReactMethod
  public void enableIncomingBanner(boolean enable) {
    TUICallKit.createInstance(mContext).enableIncomingBanner(enable);
  }

  @ReactMethod
  public void setScreenOrientation(Integer orientation) {
    TUICallKit.createInstance(mContext).setScreenOrientation(orientation);
  }

  private void destroyInstance() {
    TUICallEngine.createInstance(mContext).removeObserver(mListener);
    TUICallEngine.destroyInstance();
  }
}
