package com.tencent.cloud.rn.callkit.listener;

import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.google.gson.Gson;
import com.tencent.cloud.rn.callkit.utils.Constants;
import com.tencent.cloud.rn.callkit.utils.ParamsTransformer;
import com.tencent.qcloud.tuikit.TUICommonDefine;
import com.tencent.qcloud.tuikit.tuicallengine.TUICallDefine;
import com.tencent.qcloud.tuikit.tuicallengine.TUICallObserver;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CallEventDispatcher extends TUICallObserver {
  private static final String LISTENER_FUNC_NAME = "onListener";
  private final ReactApplicationContext mContext;

  public CallEventDispatcher(ReactApplicationContext context) {
    mContext = context;
  }

  @Override
  public void onError(int code, String msg) {
    Map<String, Object> params = new HashMap<>();
    params.put(Constants.errorCode, code);
    params.put(Constants.message, msg);
    invokeListener(Constants.CallBackFunctionName.onError, params);
  }

  @Override
  public void onCallReceived(String callerId, List<String> calleeIdList, String groupId, TUICallDefine.MediaType callMediaType, String userData) {
    Map<String, Object> params = new HashMap<>();
    int mediaType = callMediaType == TUICallDefine.MediaType.Audio ? 0 : 1;
    params.put(Constants.callerId, callerId);
    params.put(Constants.calleeIdList, calleeIdList);
    params.put(Constants.groupId, groupId);
    params.put(Constants.callMediaType, mediaType);
    params.put(Constants.userData, userData);
    invokeListener(Constants.CallBackFunctionName.onCallReceived, params);
  }

  @Override
  public void onCallCancelled(String callerId) {
    Map<String, Object> params = new HashMap<>();
    params.put(Constants.callerId, callerId);
    invokeListener(Constants.CallBackFunctionName.onCallCancelled, params);
  }

  @Override
  public void onCallBegin(TUICommonDefine.RoomId roomId, TUICallDefine.MediaType callMediaType, TUICallDefine.Role callRole) {
    Map<String, Object> params = new HashMap<>();
    Map<String, Object> roomIdMap = new HashMap<>();
    roomIdMap.put(Constants.intRoomId, roomId.intRoomId);
    roomIdMap.put(Constants.strRoomId, roomId.strRoomId);
    int mediaType = callMediaType == TUICallDefine.MediaType.Audio ? 0 : 1;

    params.put(Constants.roomId, roomIdMap);
    params.put(Constants.callMediaType, mediaType);
    params.put(Constants.callRole, ParamsTransformer.transRoleToInt(callRole));
    invokeListener(Constants.CallBackFunctionName.onCallBegin, params);
  }

  @Override
  public void onCallEnd(TUICommonDefine.RoomId roomId, TUICallDefine.MediaType callMediaType, TUICallDefine.Role callRole, long totalTime) {
    Map<String, Object> params = new HashMap<>();
    Map<String, Object> roomIdMap = new HashMap<>();
    roomIdMap.put(Constants.intRoomId, roomId.intRoomId);
    roomIdMap.put(Constants.strRoomId, roomId.strRoomId);
    int mediaType = callMediaType == TUICallDefine.MediaType.Audio ? 0 : 1;

    params.put(Constants.roomId, roomIdMap);
    params.put(Constants.callMediaType, mediaType);
    params.put(Constants.callRole, ParamsTransformer.transRoleToInt(callRole));
    params.put(Constants.totalTime, totalTime);
    invokeListener(Constants.CallBackFunctionName.onCallEnd, params);
  }

  @Override
  public void onUserReject(String userId) {
    Map<String, Object> params = new HashMap<>();
    params.put(Constants.userId, userId);
    invokeListener(Constants.CallBackFunctionName.onUserReject, params);
  }

  @Override
  public void onUserNoResponse(String userId) {
    Map<String, Object> params = new HashMap<>();
    params.put(Constants.userId, userId);
    invokeListener(Constants.CallBackFunctionName.onUserNoResponse, params);
  }

  @Override
  public void onUserLineBusy(String userId) {
    Map<String, Object> params = new HashMap<>();
    params.put(Constants.userId, userId);
    invokeListener(Constants.CallBackFunctionName.onUserLineBusy, params);
  }

  @Override
  public void onUserJoin(String userId) {
    Map<String, Object> params = new HashMap<>();
    params.put(Constants.userId, userId);
    invokeListener(Constants.CallBackFunctionName.onUserJoin, params);
  }

  @Override
  public void onCallMediaTypeChanged(TUICallDefine.MediaType oldCallMediaType, TUICallDefine.MediaType newCallMediaType) {
    Map<String, Object> params = new HashMap<>();
    int oldMediaType = oldCallMediaType == TUICallDefine.MediaType.Audio ? 0 : 1;
    int newMediaType = newCallMediaType == TUICallDefine.MediaType.Audio ? 0 : 1;
    params.put(Constants.oldCallMediaType, oldMediaType);
    params.put(Constants.newCallMediaType, newMediaType);
    invokeListener(Constants.CallBackFunctionName.onCallMediaTypeChanged, params);
  }

  @Override
  public void onKickedOffline() {
    invokeListener(Constants.CallBackFunctionName.onKickedOffline, null);
  }

  @Override
  public void onUserSigExpired() {
    invokeListener(Constants.CallBackFunctionName.onUserSigExpired, null);
  }

  private void invokeListener(String event, Object params) {
    Gson gson = new Gson();
    WritableMap backParams = Arguments.createMap();
    backParams.putString("type", event);
    backParams.putString("params", gson.toJson(params));
    sendEvent(backParams);
  }

  private void sendEvent(@Nullable WritableMap params) {
    mContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(LISTENER_FUNC_NAME, params);
  }
}
